#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Trade range bars. Price above/below 100 EMA, two consecutive up / down candles and CCI above -50/below 50.
    /// </summary>
    [Description("Trade range bars. Price above/below 100 EMA, two consecutive up / down candles and CCI above -50/below 50.")]
    public class RangePullback : Indicator
    {
        #region Variables
        // Wizard generated variables
        // User defined variables (add any user defined variables below)
		bool mailAlert = true;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Overlay				= true;
			Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Line, "Plot0"));
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBars[0] < 5)
				return;
			
            // Longs
			if (Close[0] > EMA(100)[0] &&
				Close[0] > Open[0] &&
				Close[1] > Open[1] &&
				Close[2] < Open[2] &&
				CCI(7)[0] > -50 &&
				CCI(7)[2] < -50)
			{
				DrawArrowUp(Guid.NewGuid().ToString(), true, 0, Low[0] - 2*TickSize, Color.Blue);
				if (mailAlert)
					SendMail("john@doe.com", "john@doe.com", "Buy "+Instrument.FullName, "");
				Plot0.Set(1);
				return;
			}
			
			// Shorts
			if (Close[0] < EMA(100)[0] &&
				Close[0] < Open[0] &&
				Close[1] < Open[1] &&
				Close[2] > Open[2] &&
				CCI(7)[0] < 50 &&
				CCI(7)[2] > 50)
			{
				DrawArrowDown(Guid.NewGuid().ToString(), true, 0, High[0] + 2*TickSize, Color.Cyan);
				if (mailAlert)
					SendMail("john@doe.com", "john@doe.com", "Sell "+Instrument.FullName, "");
				Plot0.Set(-1);
				return;
			}
			
			Plot0.Set(0);
				
        }

        #region Properties
		public DataSeries Plot0
        {
            get { return Values[0]; }
        }
		
		[Description("Toggle mail alert")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Toggle mail alert")]
		public bool MailAlert
		{
			get { return mailAlert; }
			set { mailAlert = value; }
		} 

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private RangePullback[] cacheRangePullback = null;

        private static RangePullback checkRangePullback = new RangePullback();

        /// <summary>
        /// Trade range bars. Price above/below 100 EMA, two consecutive up / down candles and CCI above -50/below 50.
        /// </summary>
        /// <returns></returns>
        public RangePullback RangePullback(bool mailAlert)
        {
            return RangePullback(Input, mailAlert);
        }

        /// <summary>
        /// Trade range bars. Price above/below 100 EMA, two consecutive up / down candles and CCI above -50/below 50.
        /// </summary>
        /// <returns></returns>
        public RangePullback RangePullback(Data.IDataSeries input, bool mailAlert)
        {
            if (cacheRangePullback != null)
                for (int idx = 0; idx < cacheRangePullback.Length; idx++)
                    if (cacheRangePullback[idx].MailAlert == mailAlert && cacheRangePullback[idx].EqualsInput(input))
                        return cacheRangePullback[idx];

            lock (checkRangePullback)
            {
                checkRangePullback.MailAlert = mailAlert;
                mailAlert = checkRangePullback.MailAlert;

                if (cacheRangePullback != null)
                    for (int idx = 0; idx < cacheRangePullback.Length; idx++)
                        if (cacheRangePullback[idx].MailAlert == mailAlert && cacheRangePullback[idx].EqualsInput(input))
                            return cacheRangePullback[idx];

                RangePullback indicator = new RangePullback();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.MailAlert = mailAlert;
                Indicators.Add(indicator);
                indicator.SetUp();

                RangePullback[] tmp = new RangePullback[cacheRangePullback == null ? 1 : cacheRangePullback.Length + 1];
                if (cacheRangePullback != null)
                    cacheRangePullback.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheRangePullback = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Trade range bars. Price above/below 100 EMA, two consecutive up / down candles and CCI above -50/below 50.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RangePullback RangePullback(bool mailAlert)
        {
            return _indicator.RangePullback(Input, mailAlert);
        }

        /// <summary>
        /// Trade range bars. Price above/below 100 EMA, two consecutive up / down candles and CCI above -50/below 50.
        /// </summary>
        /// <returns></returns>
        public Indicator.RangePullback RangePullback(Data.IDataSeries input, bool mailAlert)
        {
            return _indicator.RangePullback(input, mailAlert);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Trade range bars. Price above/below 100 EMA, two consecutive up / down candles and CCI above -50/below 50.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RangePullback RangePullback(bool mailAlert)
        {
            return _indicator.RangePullback(Input, mailAlert);
        }

        /// <summary>
        /// Trade range bars. Price above/below 100 EMA, two consecutive up / down candles and CCI above -50/below 50.
        /// </summary>
        /// <returns></returns>
        public Indicator.RangePullback RangePullback(Data.IDataSeries input, bool mailAlert)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.RangePullback(input, mailAlert);
        }
    }
}
#endregion
